/********************************************************************
*               MINI-COMPILADOR PARA O MINI-PASCAL                  *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                              *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*          COMPILADORES: Prof. Mrio Luis R. Oliveira               *
********************************************************************/

/* 
    OBSERVAES:
        + A classe clLogErros deve conter as funcoes Print() e GetNroErros()
        + Este analisador lxico no  "case-sensitive"
        + O procedimento para tratamento de erros foi o "MODO PNICO", descartando
          uma entrada invlida at um carcter sincronizador
        + Este analisador foi montado seguindo os autmatos descritos na apostila
          da matria de compiladores da Universidade Federal de Lavras.
    
    MTODOS:    
        + Clanalisadorlexico(cllista<cltoken> &umatabeladesimbolos, Cllogerros &logerros);
        + ~clanalisadorlexico();
        + Bool Analisar(string Nomedoarquivo);

        - Void Inserirtoken(string Umatomo, Flags Tipo, Int Nrodalinha);
        - Void Normalizar(char &caracter);
        - Bool Criarautomato();
        - Bool Ehletra(char C);
        - Bool Ehdigito(char C);
        - Bool Ehinvalido(char C);
        - Flags Stringtoflags(string Str);
*/

#ifndef ANALISADOR_LEXICO_H
#define ANALISADOR_LEXICO_H

#include "lista.h"
#include "token.h"
#include "automato.h"
#include "logErros.h"
#include <fstream>
#include "ctype.h"

using namespace std;

class clAnalisadorLexico
{
    public:
        // Construtor a partir de uma tabela de simbolos e um analisador de erros
        clAnalisadorLexico(clLista<clToken> &umaTabelaDeSimbolos, clLogErros &logErros);
        // Destrutor
        ~clAnalisadorLexico();
        // Dado um arquivo, faz a anlide do mesmo
        bool Analisar(string nomeDoArquivo);

    private:
        // Insere um token na tabela de smbolos
        void InserirToken(string umAtomo, FLAGS tipo, int nroDaLinha);
        // Normalizar um carcter significa classific-lo quanto a ser uma letra,
        // um dgito, final de linha...
        void Normalizar(char &caracter);
        //  Cria o autmato que representa o formalismo reconhecedor da gramtica
        // mini-pascal
        bool CriarAutomato();
        // Verifica se um dado char 'c'  letra ou no
        bool EhLetra(char c);
        // Verifica se um dado char 'c'  digito ou no        
        bool EhDigito(char c);
        // Verifica se um dado char 'c'  invlido para a anlise lxica
        bool EhInvalido(char c);
        // Recebe uma string e d o FLAG equivalente. Por FLAG entende-se o tipo do Token
        FLAGS StringToFLAGS(string str);
        // Referncia para a tabela de smbolos do analidador lxico. Utiliza-se lista porque 
        // apenas foi desenvolvida a fase de anlise. Caso deseje-se desenvolver um compilador
        // completo seria til a utilizao de uma tabela HASH
        clLista<clToken> *tabelaDeSimbolos;
        // Referncia ao tratador de erros. Neste trabalho o tratador de erros apenas recebe
        // um dado erro e o coloca na sada padro do sistema. 
        clLogErros *erros;
        // Referncia ao autmato utilizado pelo analisador lxico
        clAutomato<char> *automato;
};

// Construtor a partir de uma tabela de simbolos e um analisador de erros
clAnalisadorLexico::clAnalisadorLexico(clLista<clToken> &umaTabelaDeSimbolos, clLogErros &logErros)
{
    tabelaDeSimbolos = &umaTabelaDeSimbolos;
    erros = &logErros;
    automato = NULL;
}

// Destrutor
clAnalisadorLexico::~clAnalisadorLexico()
{
    delete automato;
}

// Dado um arquivo, faz a anlide do mesmo
// Varre o arquivo carcter a caracter procurando montar os tokens
// aceitos pela linguagem
bool clAnalisadorLexico::Analisar(string nomeDoArquivo)
{
    // Cria o formalismo reconhecedor
    if (CriarAutomato())
    {
        // Ser o caracter lido do arquivo
        char caracter;
        // Sada do autmato
        char saida[36];
        // Classificao quanto a ser PALAVRA, CONSTANTE, MAIOR...
        FLAGS tipo;
        // Token formado
        string token;
        // Arquivo sendo analisado
        ifstream arquivo(nomeDoArquivo.c_str());

        // Se o arquivo no existe aborta-se a anlise
        if(!arquivo)
        {
            cerr << "ERRO: O arquivo " << nomeDoArquivo << " nao pode ser aberto. "
                 << "Impossivel fazer a analise lexica." << endl;
            return false;
        } 
        else
        {
            cout << "Fazendo Analise Lexica  ...";
            int nroDaLinha = 1;
            char caracterAntigo;
            token = "";
            // Para cada token reinicializa-se o autmato
            automato->ReinicializarAutomato();
            do
            {
                // Enquanto o estado do autmato no for final no se formou um token
                while(!automato->GetEstadoAtualEhFinal())
                {
                    if (arquivo.peek() == EOF)
                        caracter = EOF;
                    else
                        arquivo.get(caracter);

                    caracterAntigo = caracter;
                    token += caracter;
                    Normalizar(caracter);
                    automato->ProcessarSimbolo(&caracter, saida);
                }

                // O autmato trabalha com uma sada interpretada pelo programa /
                // levando-se em conta apenas o seu primeiro carcter
                tipo = static_cast<FLAGS>(saida[1]);

                // Retira o espao do token
                if (tipo == PALAVRA || tipo == CONSTANTE || tipo == COMENTARIO
                    || tipo == MENOR || tipo == MAIOR || tipo == DPONTOS)
                {
                    token.resize(token.length()-1);
                }
                else
                {
                    if(caracterAntigo != 10)
                        caracterAntigo = ' ';
                }

                // Foi encontrado algum erro no token sendo montado
                if (saida[1] == '2')
                {
                    erros->Log("Char constante invalido ou terminador de string ou de comentario nao encontrado");
                    tipo = FIM;
                }

                // Insere o token na tabela de smbolos
                if(tipo != FIM && saida[1] != '1' && !token.empty())
                {
                    InserirToken(token, tipo, nroDaLinha);
                }
                // Ignora as linhas de comentrio
                if (tipo == COMENTARIO)
                {
                    for(int i = 0; i < token.length(); i++)
                    {
                        if (token[i] == '\n')
                            nroDaLinha++;
                    }
                }
                // Controla as linhas em que os tokens aparecem
                // til para o analisador de erros
                if(caracterAntigo == 10)
                    nroDaLinha++;

                // Verifica se foi encontrado o fim do cdigo sendo analisado
                if (caracter == 'e')
                    tipo = FIM;

                // Preparando para iniciar anlise de outro token
                if (caracterAntigo != ' ' && caracterAntigo != '\n' && caracterAntigo != '\t'
                    && caracterAntigo != 13 && caracterAntigo != 10 && tipo != COMENTARIO)
                {

                    automato->ReinicializarAutomato();
                    caracter = caracterAntigo;
                    token = caracter;
                    Normalizar(caracter);
                    automato->ProcessarSimbolo(&caracter, saida);
                }
                else
                {
                    token = "";
                    automato->ReinicializarAutomato();
                }
            } while(tipo != FIM);

            // Insere o EOF na tabela de smbolos como modo de se saber quando 
            // este termina
            InserirToken("EOF", ENDOFFILE, nroDaLinha);
            // Diz ao analisador de erros que a anlise lxica chegou ao seu final
            erros->Fim();
            // True significa que tudo ocorreu como se esperava
            return true;
        }
    }
    else
        cerr << "ERRO INTERNO #005. \nComo nao foi possivel criar automato, impossivel fazer analise lexica\n" << endl;
}

// Insere um dado token na tabela de smbolos.
void clAnalisadorLexico::InserirToken(string umAtomo, FLAGS tipo, int nroDaLinha)
{
    // Declara o token a ser inserido
    clToken token;
    // Cria o atomo
    string atomo(umAtomo);
    // Passa todas as letras do atomo para minusculas. O analisador lexico nao 
    //  'case sensitive'
    for (int i = 0; i < atomo.length(); i++)
            atomo[i] = tolower(atomo[i]);

    // Verifica o tipo do atomo para insero correta do token
    switch(tipo)
    {
        case NENHUMA :
            cerr << "ERRO INTERNO #006 nao esperado. \nTipo de token identificado incorretamente\n" << endl;
            break;
        case PALAVRA :
            if(StringToFLAGS(atomo) != NENHUMA)
            {
                token.SetTodos(atomo, StringToFLAGS(atomo), atomo, nroDaLinha);
            }
            else
                token.SetTodos(atomo, ID, atomo, nroDaLinha);
            break;
        case CONSTANTE :
            token.SetTodos(atomo, tipo, atomo, nroDaLinha);
            break;
        case CADEIA :
            token.SetTodos(atomo, tipo, atomo.substr(1,atomo.length()-2), nroDaLinha);
            break;
        case MAIS :
            token.SetTodos(atomo, OPAD, "mais", nroDaLinha);
            break;
        case MENOS :
            token.SetTodos(atomo, OPAD, "menos", nroDaLinha);
            break;
        case VEZES :
            token.SetTodos(atomo, OPMULT, "vezes", nroDaLinha);
            break;
        case DIV :
            token.SetTodos(atomo, OPMULT, "div", nroDaLinha);
            break;
        case OPNEG :
            token.SetTodos(atomo, tipo, "negacao", nroDaLinha);
            break;
        case IGUAL :
            token.SetTodos(atomo, OPREL, "igual", nroDaLinha);
            break;
        case MENORIGUAL :
            token.SetTodos(atomo, OPREL, "menorIgual", nroDaLinha);
            break;
        case DIFERENTE :
            token.SetTodos(atomo, OPREL, "diferente", nroDaLinha);
            break;
        case MENOR :
            token.SetTodos(atomo, OPREL, "menor", nroDaLinha);
            break;
        case MAIORIGUAL :
            token.SetTodos(atomo, OPREL, "maiorIgual", nroDaLinha);
            break;
        case MAIOR :
            token.SetTodos(atomo, OPREL, "maior", nroDaLinha);
            break;
        case PVIRG :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case PONTO :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case VIRG :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case ABREPAR :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case FECHAPAR :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case INVALIDO :
            // Quando um caracter invalido esta junto com outros caracteres ele 
            // descarta o invalido e considera o resto como token, (um antes e um depois). 
            // Ex ca&sa gera os tokens 'ca' e 'sa'
            erros->Log(atomo, nroDaLinha);
            break;
        case COMENTARIO :
            break;
        case ATRIB :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case DPONTOS :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
        case CHAR :
            token.SetTodos(atomo, tipo, atomo.substr(1,atomo.length()-2), nroDaLinha);
            break;
        case ENDOFFILE :
            token.SetTodos(atomo, tipo, "", nroDaLinha);
            break;
    }
    if (tipo != INVALIDO && tipo != COMENTARIO)
    {
        tabelaDeSimbolos->InserirNoFim(token);
    }
}

// Classifica um dado caracter segundo um cdigo padronizado de 
// reconhecimento de nosso autmato. 
// 's' -> Espao
// 'n' -> Nova linha
// 't' -> Tabulao
// 'e' -> EOF, fim de arquivo
// 'd' -> Dgito
// 'i' -> Invlido
void clAnalisadorLexico::Normalizar(char &c)
{
    if(c == ' ')
        c = 's';
    else if(c == '\n' || c == 13)
        c = 'n';
    else if(c == '\t')
        c = 't';
    else if(c == EOF)
        c = 'e';
    else if(EhLetra(c))
        c = 'l';
    else if(EhDigito(c))
        c = 'd';
    else if(EhInvalido(c))
        c = 'i';
}

// Verifica se um dado caracter  letra ou no
// Por letra entende-se pertencer ao alfabeto 
// Portugus-Brasileiro
bool clAnalisadorLexico::EhLetra(char c)
{
    return ( (c >= 65 && c <= 90) || (c >= 97 && c <= 122) );
}

// Verifica se um dado caracter  um nmero
bool clAnalisadorLexico::EhDigito(char c)
{
    return (c >= 48 && c <= 57);
}

// Verifica se um dado caracter  invlido segundo a gramtica 
// do mini-pascal
bool clAnalisadorLexico::EhInvalido(char c)
{
    return (
            ( (c <= 38) && (c != 32) && (c != 34) ) ||
            (c == 63) ||
            (c == 64) ||
            (c >= 91 && c <= 96) ||
            (c == 124) ||
            (c == 127)
           );
}

// Recebe uma string e d o FLAG equivalente. Por FLAG entende-se o tipo do Token
FLAGS clAnalisadorLexico::StringToFLAGS(string str)
{
    for (int i = 0; i < str.length(); i++)
        str[i] = tolower(str[i]);
    if(str == "programa")
        return PROGRAMA;
    else if(str == "comeco")
        return COMECO;
    else if(str == "logico")
        return LOGICO;
    else if(str == "faca")
        return  FACA;
    else if(str == "entao")
        return ENTAO;
    else if(str == "fim")
        return FIM;
    else if(str == "falso")
        return FALSO;
    else if(str == "se")
        return SE;
    else if(str == "inteiro")
        return INTEIRO;
    else if(str == "real")
        return REAL;
    else if(str == "verdadeiro")
        return VERDADEIRO;
    else if(str == "enquanto")
        return ENQUANTO;
    else if(str == "variaveis")
        return VARIAVEIS;
    else if(str == "senao")
        return SENAO;
    else if(str == "char")
        return  CHAR;
    else if(str == "escreva")
        return ESCREVA;
    else if(str == "leia")
        return LEIA;
    else if(str == "constante")
        return CONSTANTE;
    else if(str == "cadeia")
        return CADEIA;
    else if(str == "e" || str == "ou")
        return OPLOG;
    else
        return NENHUMA;
}

// Cria o automato a partir da base de dados
bool clAnalisadorLexico::CriarAutomato()
{
    string str;

    long int identificador;
    bool ehFinal;
    char saida[36];


    // Objeto usado para se trabalhar com os arquivos da base de dados
    ifstream arquivo("estados.lex");

    // Se os arquivos foram abertos com exito cria o automato a partir deles
    if (!arquivo)
    {
        cerr << "ERRO INTERNO #001: Nao foi possivel criar ou abrir arquivo estados.lex. "
             << "\nImpossivel fazer a analise lexica\n"
             << "Reporte este BUG aos desenvolvedores" << endl;
        automato = NULL;
        return false;
    }
    else
    {
        arquivo >> identificador >> ehFinal;
        arquivo.getline(saida, sizeof(saida));
        automato = new clAutomato<char>(identificador, ehFinal, saida);

        while(arquivo >> identificador >> ehFinal && arquivo.getline(saida, sizeof(saida)))
        {
            if (!automato->InserirEstado(identificador, ehFinal, saida))
                cout << "ERRO INTERNO #002 nao esperado. Arquivo estados.lex invalido"  << endl;
        }
    }

    ifstream ligacoes("producoes.lex");
    int idPartida, idChegada;
    char simbolo;

    if (!ligacoes)
    {
        cerr << "ERRO INTERNO #003: Nao foi possvel criar ou abrir arquivo producoes.lex. "
             << "\nImpossivel fazer a analise lexica\n"
             << "Reporte este BUG aos desenvolvedores" << endl;
        automato = NULL;
        return false;
    }
    else
    {
        while(ligacoes >> idPartida >> idChegada >> simbolo)
        {
            if (!automato->InserirProducao(idPartida, idChegada, &simbolo))
                cout << "ERRO INTERNO #004.  Arquivo producoes.lex invalido" << endl;
        }
    }
    return true;
}

#endif

