/********************************************************************
*               MINI-COMPILADOR PARA O MINI-PASCAL                  *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                              *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*          COMPILADORES: Prof. Mrio Luis R. Oliveira               *
********************************************************************/

/* 
    OBSERVAES:
        + A classe clLogErros deve conter as funcoes Print() e GetNroErros()
        + Todos os autmatos aqui utilizados como formalismo reconhecedor da 
          linguagem mini-pascal esto descritos na apostila da disciplina de
          compiladores da Universidade Federal de Lavras

    MTODOS:
        + clAnalisadorSintatico(clLista<clToken> &umaTabelaDeSimbolos, clLogErros &logErros);
        + ~clAnalisadorSintatico();
        + bool Analisar();

        - void ExecutarFuncao(string funcao);
        - bool CriarAutomatos();
        - clAutomato<FLAGS> *CriarEstados(ifstream &arquivo);
        - void CriarLigacoes(ifstream &arquivo, clAutomato<FLAGS> *automato);
        - void PercorrerAutomato(clAutomato<FLAGS> *automato);
        - void Programa();
        - void Declaracoes();
        - void ListaDeDeclaracoes();
        - void DeclaracaoDeTipo();
        - void ListaDeIds(); 
        - void Tipo(); 
        - void C_comp();
        - void ListaDeComandos();
        - void Comando();
        - void Se();
        - void Enquanto();
        - void Read();
        - void Write();
        - void ListaDoWrite();
        - void ElementoDoWrite();
        - void Atribuicao();
        - void Expressao();
        - void ParteDaExpressao();
        - void Termo();
        - void Fator();
        - void AnaliseSemanticaID(clToken token);
        - void AnaliseSemanticaTipo(FLAGS tipo);
        - void AnaliseSemanticaCasamento();
*/

#ifndef ANALISADOR_SINTATICO_H
#define ANALISADOR_SINTATICO_H

using namespace std;

#include "lista.h"
#include "token.h"
#include "automato.h"
#include "logErros.h"
#include <fstream>
#include "ctype.h"
#include "identificador.h"

// Utilizado na anlise semntica 
enum STATUS { NOMEANDO_PROGRAMA = 200, DECLARANDO_VAR, VAR_DECLARADAS, EXPRESSAO_LOGICA, };

class clAnalisadorSintatico
{
    public:
        // Construtor padro. Recebe uma referncia para a tabela de smbolos a qual deve
        // seguir e tambm qual  o objeto tratador de erros
        clAnalisadorSintatico(clLista<clToken> &umaTabelaDeSimbolos, clLogErros &logErros);
        // Destrutor
        ~clAnalisadorSintatico();
        // Faz a anlise sinttica
        bool Analisar();
    private:
        // Executa uma funo representando a ao de um dado automato
        void ExecutarFuncao(string funcao);
        // Cria os autmatos que sero utilizados como formalismo reconhecedor
        bool CriarAutomatos();
        // Cria os estados necessrios ao autmato
        clAutomato<FLAGS> *CriarEstados(ifstream &arquivo);
        // Cria as ligaes necessrias ao autmato
        void CriarLigacoes(ifstream &arquivo, clAutomato<FLAGS> *automato);
        // Percorre o autmato tentando reconhecer algum formalismo
        void PercorrerAutomato(clAutomato<FLAGS> *automato);
        // Cada uma das funes seguintes so relacionadas ao autmato comentado  direita. Maiores detalhes podem ser
        // encontrados na apostila da disciplina de compiladores descrita logo no incio desta classe
        void Programa(); // <prog>
        void Declaracoes(); // <decls>
        void ListaDeDeclaracoes(); // <list_decl>
        void DeclaracaoDeTipo(); // <decl_tip>
        void ListaDeIds(); // <list_id>
        void Tipo(); // <tip>
        void C_comp(); // <c_comp>
        void ListaDeComandos(); // <list_cmd>
        void Comando(); // <cmd>
        void Se(); // <c_if>
        void Enquanto(); // <c_while>
        void Read(); // <c_read>
        void Write(); // <c_write>
        void ListaDoWrite(); // <list_w>
        void ElementoDoWrite(); // <elem_w>
        void Atribuicao(); // <c_atrib>
        void Expressao(); // <expr>
        void ParteDaExpressao(); // <e_simp>
        void Termo(); // <term>
        void Fator(); // <fat>

        // Faz a anlise semntica
        // Verifica se um ID j foi declarado ao ser utilizado
        void AnaliseSemanticaID(clToken token);
        // Verifica o tipo de um FLAG especfico
        void AnaliseSemanticaTipo(FLAGS tipo);
        // Verifica compatibilidade de tipos em alguns tipos de atribuies
        void AnaliseSemanticaCasamento();

        // Automatos que descrevem o formalismo sintatico da linguagem mini-pascal
        clAutomato<FLAGS> *prog;
        clAutomato<FLAGS> *decls;
        clAutomato<FLAGS> *list_decl;
        clAutomato<FLAGS> *decl_tip;
        clAutomato<FLAGS> *list_id;
        clAutomato<FLAGS> *tip;
        clAutomato<FLAGS> *c_comp;
        clAutomato<FLAGS> *list_cmd;
        clAutomato<FLAGS> *cmd;
        clAutomato<FLAGS> *c_if;
        clAutomato<FLAGS> *c_while;
        clAutomato<FLAGS> *c_read;
        clAutomato<FLAGS> *c_write;
        clAutomato<FLAGS> *list_w;
        clAutomato<FLAGS> *elem_w;
        clAutomato<FLAGS> *c_atrib;
        clAutomato<FLAGS> *expr;
        clAutomato<FLAGS> *e_simp;
        clAutomato<FLAGS> *term;
        clAutomato<FLAGS> *fat;

        // teis para anlise semntica
        STATUS status;
        // Guarda se uma dada declarao fechou os parnteses
        bool fechouParenteses;
        // Guarda se uma dada declarao  uma expresso booleana
        bool ehBooleano;
        // Guarda o nome do programa sendo analisado neste momento
        string nomeDoPrograma;
        // Lista dos ids
        clLista<clIdentificador> listaDeIds;
        // Referncia para o primeiro token
        clNohLista<clToken> *nohToken;
        // Referncia para o incio da tabela de smbolos
        clLista<clToken> *tabelaDeSimbolos;
        // Referncia a tratador de erros
        clLogErros *erros;
};

// Construtor do analisador sinttico
clAnalisadorSintatico::clAnalisadorSintatico(clLista<clToken> &umaTabelaDeSimbolos, clLogErros &logErros)
{
    tabelaDeSimbolos = &umaTabelaDeSimbolos;
    erros = &logErros;
    prog = NULL;
    decls = NULL;
    list_decl = NULL;
    decl_tip = NULL;
    list_id = NULL;
    tip = NULL;
    c_comp = NULL;
    list_cmd = NULL;
    cmd = NULL;
    c_if = NULL;
    c_while = NULL;
    c_read = NULL;
    c_write = NULL;
    list_w = NULL;
    elem_w = NULL;
    c_atrib = NULL;
    expr = NULL;
    e_simp = NULL;
    term = NULL;
    fat = NULL;
    ehBooleano = false;
}

// Destrutor do analisador sinttico. As alocaes dinmicas so tratadas
// aqui. 
clAnalisadorSintatico::~clAnalisadorSintatico()
{
    delete prog;
    delete decls;
    delete list_decl;
    delete decl_tip;
    delete list_id;
    delete tip;
    delete c_comp;
    delete list_cmd;
    delete cmd;
    delete c_if;
    delete c_while;
    delete c_read;
    delete c_write;
    delete list_w;
    delete elem_w;
    delete c_atrib;
    delete expr;
    delete e_simp;
    delete term;
    delete fat;
}

// Inicia a anlise sinttica
bool clAnalisadorSintatico::Analisar()
{
    cout << "Fazendo Analise Sintatica...";
    // Verifica se pode criar os autmatos
    if(CriarAutomatos())
    {
        // Referncia a um noh de lista
        clNohLista<clToken> *nohAux;

        // Primeiro token da tabela de smbolos
        nohToken = tabelaDeSimbolos->Primeiro();

        // Procurando nome do programa
        nohAux = nohToken;

        while(nohAux->Valor().GetTipo() != PROGRAMA && nohAux->Valor().GetTipo() != ENDOFFILE)
            nohAux = nohAux->Proximo();
        if (nohAux->Valor().GetTipo() == PROGRAMA && nohAux->Proximo()->Valor().GetTipo() == ID)
            nomeDoPrograma = nohAux->Proximo()->Valor().GetAtomo();
        else
            nomeDoPrograma = "";

        // Funes auxiliares
        Programa();
        // Repare que a anlise semntica no  independente da anlise sinttica
        AnaliseSemanticaCasamento();
        // Aviso ao analisador de erros que acabou a anlise sinttica
        erros->Fim();
        return true;
    }
    else
        cerr << "ERRO INTERNO #007: Como nao foi possivel criar os automatos, impossivel fazer Analise Sintatica" << endl;
}

// Percorre o autmato em busca de um formalismo a ser reconhecido
void clAnalisadorSintatico::PercorrerAutomato(clAutomato<FLAGS> *automato)
{
    char saidaDoEstado[36];
    string saidaDaTransicao, strErro;
    FLAGS avanca = AVANCA;
    FLAGS coringa = CORINGA;
    FLAGS tipo;

    // Reinicializa o autmato permitindo execuo de outra anlise
    automato->ReinicializarAutomato();
    // Enquanto no se chegar a um estado final...
    while(!automato->GetEstadoAtualEhFinal() && nohToken != NULL)
    {
        saidaDaTransicao = "nada";
        // processar smbolo a smbolo...
        if (automato->TentarProcessarSimbolo(&avanca, saidaDoEstado, saidaDaTransicao))
        {
            // cada funo disponvel
            ExecutarFuncao(saidaDaTransicao);
        }
        else
        {
            tipo = nohToken->Valor().GetTipo();
            if (automato->TentarProcessarSimbolo(&tipo, saidaDoEstado, saidaDaTransicao))
            {
                // Indica que pede um novo atomo
                if (saidaDaTransicao[1] == '0') 
                {
                    if (nohToken->Valor().GetTipo() == ID)
                        AnaliseSemanticaID(nohToken->Valor());
                    nohToken = nohToken->Proximo();
                }
            }
            else
            {
                if (automato->TentarProcessarSimbolo(&coringa, saidaDoEstado, saidaDaTransicao))
                {
                    // 1 indica que nao eh Esperado(), nem novoAtomo
                    if (saidaDaTransicao[1] != '1')
                    {
                        if (saidaDaTransicao[1] != '0')
                        {
                            strErro = "'" + nohToken->Valor().GetAtomo() + "' : " + saidaDaTransicao;
                            erros->Log(strErro, nohToken->Valor().GetLinha());
                        }
                        nohToken = nohToken->Proximo();
                    }
                }
                else
                {
                   cerr << "ERRO INTERNO #008 nao esperado: Erro na descricao do automato de analise sintatica" << endl;
                   cerr << "Arquivo 'producoes.sin' invalido" << endl;
                }
            }
        }
    }
}

// Executa uma funo em especfico. Representa a ao de um autmato
void clAnalisadorSintatico::ExecutarFuncao(string funcao)
{
    switch (funcao[1])
    {
        case 'a' :
            // Fase de declarao no cdigo fonte
            Declaracoes();
            break;
        case 'b' :
            // Lista das declaraes
            ListaDeDeclaracoes();
            break;
        case 'c' :
            // Tipos das declaraes
            DeclaracaoDeTipo();
            break;
        case 'd' :
            // Lista dos ids nas declaraes            
            ListaDeIds();
            break;
        case 'e' :
            // Tipos de cada um
            Tipo();
            break;
        case 'f' :
            C_comp();
            break;
        case 'g' :
            ListaDeComandos();
            break;
        case 'h' :
            Comando();
            break;
        case 'i' :
            Se();
            break;
        case 'j' :
            Enquanto();
            break;
        case 'l' :
            Read();
            break;
        case 'm' :
            Write();
            break;
        case 'n' :
            ListaDoWrite();
            break;
        case 'o' :
            ElementoDoWrite();
            break;
        case 'p' :
            Atribuicao();
            break;
        case 'q' :
            Expressao();
            break;
        case 'r' :
            ParteDaExpressao();
            break;
        case 's' :
            Termo();
            break;
        case 't' :
            Fator();
            break;
        case 'u' :
            // Anlise semntica, verifica o casamento de parnteses bem como 
            // a concatenao de vrias expresses lgicas dentro de 
            // se's e enquanto's
            if (status == EXPRESSAO_LOGICA && !fechouParenteses && !ehBooleano)
                erros->Log("Funcao SE ou ENQUANTO espera uma expressao logica");
            ehBooleano = false;
            break;
        case 'v' :
            // Serve para anlise semntica. Fase em que as variveis j foram declaradas
            status = VAR_DECLARADAS;
            break;
        case 'x' :
            // Os parnteses da expresso atual ainda no foram fechados
            fechouParenteses = false;
            break;
        case 'y' :
            // Os parnteses da expresso atual foram fechados
            fechouParenteses = true;
            break;
        case 'z' :
            // Anlise semntica. Verifica expresses lgicas como um todo
            if (status == EXPRESSAO_LOGICA)
            {
                if (nohToken->Valor().GetTipo() == VERDADEIRO || nohToken->Valor().GetTipo() == FALSO)
                {
                    ehBooleano = true;
                }
                else if (nohToken->Valor().GetTipo() == ID)
                {
                    clNohLista<clIdentificador> *noh;
                    noh = listaDeIds.Primeiro();

                    while (noh != NULL && (noh->Valor().GetNome() != nohToken->Valor().GetAtomo()))
                        noh = noh->Proximo();

                    if (noh!= NULL && noh->Valor().GetTipo() == LOGICO)
                        ehBooleano = true;
                }
            }
            nohToken = nohToken->Proximo();
            break;
        default :
            cerr << "ERRO INTERNO #009 inesperado. Possivelmente o arquivo 'producoes.sin' esta invalido" << endl;
    }
}

void clAnalisadorSintatico::Programa() // <prog>
{
    status = NOMEANDO_PROGRAMA;
    fechouParenteses = false;

    PercorrerAutomato(prog->CopiarAutomato());
}

void clAnalisadorSintatico::Declaracoes() // <decls>
{
    status = DECLARANDO_VAR;
    PercorrerAutomato(decls->CopiarAutomato());
    status = VAR_DECLARADAS;
}

void clAnalisadorSintatico::ListaDeDeclaracoes() // <list_decl>
{
    PercorrerAutomato(list_decl->CopiarAutomato());
}

void clAnalisadorSintatico::DeclaracaoDeTipo() // <decl_tip>
{
    PercorrerAutomato(decl_tip->CopiarAutomato());
}

void clAnalisadorSintatico::ListaDeIds() // <list_id>
{
    PercorrerAutomato(list_id->CopiarAutomato());
}

// Verifica o tipo de um token
void clAnalisadorSintatico::Tipo() // <tip>
{
    switch (nohToken->Valor().GetTipo())
    {
        case CADEIA :
        case LOGICO :
        case INTEIRO :
        case CHAR :
        case REAL :
            AnaliseSemanticaTipo(nohToken->Valor().GetTipo());
            break;
    }
    PercorrerAutomato(tip->CopiarAutomato());
}

void clAnalisadorSintatico::C_comp() // <c_comp>
{
    PercorrerAutomato(c_comp->CopiarAutomato());
}

void clAnalisadorSintatico::ListaDeComandos() // <list_cmd>
{
    PercorrerAutomato(list_cmd->CopiarAutomato());
}

void clAnalisadorSintatico::Comando() // <cmd>
{
    PercorrerAutomato(cmd->CopiarAutomato());
}

void clAnalisadorSintatico::Se() // <c_if>
{
    status = EXPRESSAO_LOGICA;
    PercorrerAutomato(c_if->CopiarAutomato());
}

void clAnalisadorSintatico::Enquanto() // <c_while>
{
    status = EXPRESSAO_LOGICA;
    PercorrerAutomato(c_while->CopiarAutomato());
}

void clAnalisadorSintatico::Read() // <c_read>
{
    PercorrerAutomato(c_read->CopiarAutomato());
}

void clAnalisadorSintatico::Write() // <c_write>
{
    PercorrerAutomato(c_write->CopiarAutomato());
}

void clAnalisadorSintatico::ListaDoWrite() // <list_w>
{
    PercorrerAutomato(list_w->CopiarAutomato());
}

void clAnalisadorSintatico::ElementoDoWrite() // <elem_w>
{
    PercorrerAutomato(elem_w->CopiarAutomato());
}

void clAnalisadorSintatico::Atribuicao() // <c_atrib>
{
    PercorrerAutomato(c_atrib->CopiarAutomato());
}

void clAnalisadorSintatico::Expressao() // <expr>
{
    PercorrerAutomato(expr->CopiarAutomato());
}

void clAnalisadorSintatico::ParteDaExpressao() // <e_simp>
{
    PercorrerAutomato(e_simp->CopiarAutomato());
}

void clAnalisadorSintatico::Termo() // <term>
{
    PercorrerAutomato(term->CopiarAutomato());
}

void clAnalisadorSintatico::Fator() // <fat>
{
    PercorrerAutomato(fat->CopiarAutomato());
}

// Cria os autmatos a partir da base de dados
bool clAnalisadorSintatico::CriarAutomatos()
{
    ifstream arquivoDeEstados("estados.sin");

    if (!arquivoDeEstados)
    {
        cerr << "ERRO INTERNO #010: No foi possvel criar ou abrir arquivo estados.sin. "
             << "Impossivel fazer a analise sintatica";
        return false;
    }
    else
    {
        // Criao dos estados dos autmatos
        prog = CriarEstados(arquivoDeEstados);
        decls = CriarEstados(arquivoDeEstados);
        list_decl = CriarEstados(arquivoDeEstados);
        decl_tip = CriarEstados(arquivoDeEstados);
        list_id = CriarEstados(arquivoDeEstados);
        tip = CriarEstados(arquivoDeEstados);
        c_comp = CriarEstados(arquivoDeEstados);
        list_cmd = CriarEstados(arquivoDeEstados);
        cmd = CriarEstados(arquivoDeEstados);
        c_if = CriarEstados(arquivoDeEstados);
        c_while = CriarEstados(arquivoDeEstados);
        c_read = CriarEstados(arquivoDeEstados);
        c_write = CriarEstados(arquivoDeEstados);
        list_w = CriarEstados(arquivoDeEstados);
        elem_w = CriarEstados(arquivoDeEstados);
        c_atrib = CriarEstados(arquivoDeEstados);
        expr = CriarEstados(arquivoDeEstados);
        e_simp = CriarEstados(arquivoDeEstados);
        term = CriarEstados(arquivoDeEstados);
        fat = CriarEstados(arquivoDeEstados);
    }

    ifstream arquivoDeLigacoes("producoes.sin");

    if (!arquivoDeLigacoes)
    {
        cerr << "ERRO INTERNO #011: No foi possvel criar ou abrir arquivo producoes.sin. "
             << "Impossivel fazer a analise sintatica";
        return false;
    }
    else
    {
        // Criao das ligaes dos autmatos
        CriarLigacoes(arquivoDeLigacoes, prog);
        CriarLigacoes(arquivoDeLigacoes, decls);
        CriarLigacoes(arquivoDeLigacoes, list_decl);
        CriarLigacoes(arquivoDeLigacoes, decl_tip);
        CriarLigacoes(arquivoDeLigacoes, list_id);
        CriarLigacoes(arquivoDeLigacoes, tip);
        CriarLigacoes(arquivoDeLigacoes, c_comp);
        CriarLigacoes(arquivoDeLigacoes, list_cmd);
        CriarLigacoes(arquivoDeLigacoes, cmd);
        CriarLigacoes(arquivoDeLigacoes, c_if);
        CriarLigacoes(arquivoDeLigacoes, c_while);
        CriarLigacoes(arquivoDeLigacoes, c_read);
        CriarLigacoes(arquivoDeLigacoes, c_write);
        CriarLigacoes(arquivoDeLigacoes, list_w);
        CriarLigacoes(arquivoDeLigacoes, elem_w);
        CriarLigacoes(arquivoDeLigacoes, c_atrib);
        CriarLigacoes(arquivoDeLigacoes, expr);
        CriarLigacoes(arquivoDeLigacoes, e_simp);
        CriarLigacoes(arquivoDeLigacoes, term);
        CriarLigacoes(arquivoDeLigacoes, fat);
    }
    return true;
}

// Anlise semntica: verifica se um ID tem o mesmo nome
// do programa sendo analisado. Verifica se um ID no foi 
// declarado ou, por outro lado, se foi declarado mais de uma vez,
void clAnalisadorSintatico::AnaliseSemanticaID(clToken token)
{
    clIdentificador id(token.GetAtomo());
    string str("");

    if (token.GetAtomo() == nomeDoPrograma)
    {
        if (status != NOMEANDO_PROGRAMA)
            erros->Log("Uma variavel nao pode ter o mesmo nome do programa");
    }
    if (status == DECLARANDO_VAR)
    {
        if (!listaDeIds.Encontrar(id))
            listaDeIds.InserirNoFim(id);
        else
        {
            str = "Identificador '";
            str += token.GetAtomo();
            str += "' declarado mais de uma vez";
            erros->Log(str, token.GetLinha());
        }
    }
    else if (status != DECLARANDO_VAR && status != NOMEANDO_PROGRAMA)
    {
        if(!listaDeIds.Encontrar(id))
        {
            str = "Identificador '";
            str += token.GetAtomo();
            str += "' nao foi declarado";
            erros->Log(str, token.GetLinha());
        }
    }
}

// Verifica compatibilidade de tipos
void clAnalisadorSintatico::AnaliseSemanticaTipo(FLAGS tipo)
{
    clNohLista<clIdentificador> *noh;
    noh = listaDeIds.Primeiro();

    while (noh != NULL && noh->Valor().GetTipo() != NENHUMA)
        noh = noh->Proximo();

    while (noh!= NULL)
    {
        noh->ValorRef()->SetTipo(tipo);
        noh = noh->Proximo();
    }
}

// Verifica se uma atribuio  vlida
void clAnalisadorSintatico::AnaliseSemanticaCasamento()
{
    clNohLista<clToken> *nohAtual;
    clNohLista<clToken> *nohAnterior;
    FLAGS tipo;
    clIdentificador idAnterior, idAtual, idTemp;
    string str;

    nohAtual = tabelaDeSimbolos->Primeiro();
    nohAnterior = NULL;

    while (nohAtual != NULL)
    {
        while(nohAtual != NULL && nohAtual->Valor().GetTipo() != ATRIB)
        {
            nohAnterior = nohAtual;
            nohAtual = nohAtual->Proximo();
        }
        idAnterior.SetNome(nohAnterior->Valor().GetAtomo());
        idTemp = idAnterior;
        if (listaDeIds.Encontrar(idTemp))
            idAnterior = idTemp;

        if (nohAtual != NULL)
        {
            while (nohAtual != NULL && nohAtual->Valor().GetTipo() != PVIRG)
            {
                idAtual.SetTipo(NENHUMA);

                idAtual.SetNome(nohAtual->Valor().GetAtomo());
                idTemp = idAtual;
                if (listaDeIds.Encontrar(idTemp))
                    idAtual = idTemp;

                switch (idAtual.GetTipo())
                {
                    case CADEIA :
                    case LOGICO :
                    case INTEIRO :
                    case CHAR :
                    case REAL :
                        if (idAtual.GetTipo() != idAnterior.GetTipo())
                        {
                            str = "Atribuicao de tipos incompativeis, '";
                            str += idAtual.GetNome();
                            str += "' para '";
                            str += idAnterior.GetNome();
                            str += "'";
                            erros->Log(str, nohAtual->Valor().GetLinha());
                        }
                        break;
                }
                nohAtual = nohAtual->Proximo();
            }
            if (nohAtual != NULL)
            {
                nohAnterior = nohAtual;
                nohAtual = nohAtual->Proximo();
            }
        }
    }
}

// Cria os estados dos autmatos a partir da base de dados
clAutomato<FLAGS> *clAnalisadorSintatico::CriarEstados(ifstream &arquivo)
{
    clAutomato<FLAGS> *automato;
    long int identificador;
    bool ehFinal;
    char saida[36];

    arquivo >> identificador >> ehFinal;
    arquivo.getline(saida, sizeof(saida));

    automato = new clAutomato<FLAGS>(identificador, ehFinal, saida);

    while(arquivo >> identificador >> ehFinal && arquivo.getline(saida, sizeof(saida)) && identificador != 0)
    {
        if (!automato->InserirEstado(identificador, ehFinal, saida))
            cerr << "ERRO INTERNO #012 no esperado. Possivelmente o arquivo 'estados.sin' esta invalido"  << endl;
    }
    return automato;
}

// Cria as ligaes dos autmatos a partir da base de dados
void clAnalisadorSintatico::CriarLigacoes(ifstream &arquivo, clAutomato<FLAGS> *automato)
{
    int idPartida, idChegada;
    int simbolo;
    char saida[60];
    string saidaStr;
    FLAGS simboloFlag;

    while((arquivo >> idPartida >> idChegada >> simbolo && arquivo.getline(saida, sizeof(saida))) && idPartida != 0)
    {
        simboloFlag = static_cast<FLAGS>(simbolo);
        saidaStr = saida;
        if (!automato->InserirProducao(idPartida, idChegada, &simboloFlag, saidaStr))
            cerr << "ERRO INTERNO #013 no esperado. Possivelmente o arquivo 'producoes.sin' esta invalido" << endl;
    }
}

#endif

