/********************************************************************
*               MINI-COMPILADOR PARA O MINI-PASCAL                  *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                              *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*          COMPILADORES: Prof. Mrio Luis R. Oliveira               *
* ----------------------------------------------------------------- *
* Desenvolvida inicialmente para a disciplina de Linguagens Formais *
*         e autmatos. Prof. Jones Oliveira Albuquerque.            *
*                                                                   *
*  Anderson de Rezende Rocha, Flvio Luiz Alves, Jlio Csar Alves  *
*                       Rodrigo Nazar Leite                        *
********************************************************************/

/*
    OBSERVAES:
        + Define a classe que representa uma ligacao entre dois estados (uma producao)
        + A ligacao possui uma saida, caracterizando uma Maquina de Mealy
    
    MTODOS:
        + clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino = NULL);
        + clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino, string saida);
        + clLigacao<TSimbolo>(const clLigacao<TSimbolo> &ligacao);
        + ~clLigacao<TSimbolo>() {};
        + clEstado<TSimbolo> *getDestino();
        + TSimbolo *getSimbolo();
        + string getSaida();
        + bool operator == (clLigacao<TSimbolo> ligacao);
*/

#ifndef LIGACAO_H
#define LIGACAO_H

// OBSERVACAO
// A classe TSimbolo deve definir o operador ==
#include "estado.h"
#include <string>

using namespace std;

// Diretiva para o compilador saber que futuramente encontrara tal classe
template <class TSimbolo>
class clEstado;

// Definicao da classe template clLigacao
template <class TSimbolo>
class clLigacao
{
    public:
        // Construtor a partir de um simbolo e um estado destino
        clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino = NULL);
        // Construtor a partir de um simbolo, um estado destino e uma saida
        clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino, string saida);
        // Construtor de copia
        clLigacao<TSimbolo>(const clLigacao<TSimbolo> &ligacao);
        // Apenas define o destrutor
        ~clLigacao<TSimbolo>() {};
        // Retorna o destino para o qual esta ligacao esta apontando
        clEstado<TSimbolo> *getDestino();
        // Retorna o simbolo usado nesta ligacao (simbolo consumido na producao)
        TSimbolo *getSimbolo();
        // devolve a saida da producao
        string getSaida();
        // Compara uma ligacao com outra somente em relacao ao simbolo
        bool operator == (clLigacao<TSimbolo> ligacao);
    private:
        // Guarda uma referencia ao destino desta ligacao
        clEstado<TSimbolo> *destino;
        // Simbolo desta producao
        TSimbolo simbolo;
        // Saida da ligacao
        string saida;
};

// Construtor a partir de um simbolo e um estado destino
template <class TSimbolo>
clLigacao<TSimbolo>::clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino)
{
    clLigacao<TSimbolo>::simbolo = simbolo;
    clLigacao<TSimbolo>::destino = destino;
    clLigacao<TSimbolo>::saida = "";
}

// Construtor a partir de um simbolo, um estado destino e uma saida
template <class TSimbolo>
clLigacao<TSimbolo>::clLigacao<TSimbolo>(TSimbolo simbolo, clEstado<TSimbolo>* destino, string saida)
{
    clLigacao<TSimbolo>::simbolo = simbolo;
    clLigacao<TSimbolo>::destino = destino;
    clLigacao<TSimbolo>::saida = saida;
}

// Construtor de copia
template <class TSimbolo>
clLigacao<TSimbolo>::clLigacao<TSimbolo>(const clLigacao<TSimbolo> &ligacao)
{
    simbolo = ligacao.simbolo;
    destino = ligacao.destino;
    saida = ligacao.saida;
}

// Retorna o destino desta ligacao
template <class TSimbolo>
clEstado<TSimbolo> *clLigacao<TSimbolo>::getDestino()
{
    return destino;
}

// Retorna o simbolo desta ligacao
template <class TSimbolo>
TSimbolo *clLigacao<TSimbolo>::getSimbolo()
{
    return &simbolo;
}

// Duas ligacoes sao iguais de acordo com o seus simbolos
// Por Exemplo:
// se duas ligacoes tem simbolo 'a' e destinos q1 e q3, elas sao consideradas iguais
template <class TSimbolo>
bool clLigacao<TSimbolo>::operator == (clLigacao<TSimbolo> ligacao)
{
    return (*getSimbolo() == *(ligacao.getSimbolo()));
}

// Devolve a saida da ligacao
template <class TSimbolo>
string clLigacao<TSimbolo>::getSaida()
{
    return saida;
}

#endif


