/********************************************************************
*               MINI-COMPILADOR PARA O MINI-PASCAL                  *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                              *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*          COMPILADORES: Prof. Mrio Luis R. Oliveira               *
********************************************************************/

/*
    OBSERVAES:
        + Lista  utilizada para construo da tabela de smbolos do
          compilador e pelo automato
    
    MTODOS DISPONVEIS:          
        + clLista();           
        + clLista(const clLista<TValor> &lista); 
        + ~clLista();           
        + void clLista<TValor>::Limpar();
        + clNohLista<TValor> *Primeiro() const;
        + void InserirNoFim(const TValor &val);
        + bool Encontrar(TValor& valor);
        + void Imprimir();
*/

#ifndef CLLISTA_H
#define CLLISTA_H

#include <iostream>
#include "nohlista.h"

using namespace std;

// Definida como um template para que se possa criar uma lista genrica
template <class TValor>
class clLista
{
    public:
        // Construtor padro
        clLista();           
        // Construtor de cpia
        clLista(const clLista<TValor> &lista); 
        // Destrutor
        ~clLista();           
        // Elimina todos os elementos da lista
        void clLista<TValor>::Limpar();
        // Devolve o primeiro elemento da lista
        clNohLista<TValor> *Primeiro() const;
        // Insere o elemento 'val' no fim da lista
        void InserirNoFim(const TValor &val);
        // Encontra um valor na lista e atualiza por dentro
        bool Encontrar(TValor& valor);

        // Imprime na sada padro a lista atual. Para funcionar a 
        // classe 'TValor' precisa dizer como pode ser impressa
        void Imprimir();
    protected:
        // Referncia ao primeiro elemento da lista
        clNohLista<TValor> *primeiro;   
};

// Construtor padro
template <class TValor>
clLista<TValor>::clLista()
{
    primeiro = NULL;
}

// Construtor de cpia
template <class TValor>
clLista<TValor>::clLista(const clLista<TValor> &lista)
{
    clNohLista<TValor> *noh;

    // criar a lista vazia...
    primeiro = NULL;
    noh = lista.Primeiro();

    // ... e copiar cada elemento
    while (noh != NULL)
    {
        InserirNoFim(noh->Valor());
        noh = noh->Proximo();
    }
}

// Percorre toda a lista em busca do elemento passado. Retorna TRUE
// se o mesmo foi encontrado e tambm atualiza a referncia do mesmo 
// por dentro
template <class TValor>
bool clLista<TValor>::Encontrar(TValor& valor)
{
    if (primeiro != NULL)
    {
        clNohLista<TValor> *nohAux;
        nohAux = primeiro;
        while (nohAux != NULL)
        {
            if (valor == nohAux->Valor())
            {
                valor = nohAux->Valor();
                return true;
            }
            else
                nohAux = nohAux->Proximo();
        }
    }
    return false;
}

// Limpa a lista
template <class TValor>
void clLista<TValor>::Limpar()
// Apaga todos elementos da lista, tornando-a vazia
{
    clNohLista<TValor> *aux;

    // destruir todos os nohs da lista
    aux = primeiro;
    while(aux != NULL)
    {
        primeiro = aux->Proximo();
        delete aux;
        aux = primeiro;
    }
}


// Remove todos os nohs
template <class TValor>
clLista<TValor>::~clLista()
{
    Limpar(); 
}

// Referncia ao primeiro elemento da lista
template <class TValor>
clNohLista<TValor> *clLista<TValor>::Primeiro() const
{
    return primeiro;
}

// Insere UMA COPIA do valor passado no fim da lista
template <class TValor>
void clLista<TValor>::InserirNoFim(const TValor &val)
{
    clNohLista<TValor> *atual;
    clNohLista<TValor> *anterior;

    atual = primeiro;
    anterior = NULL;

    while (atual != NULL)
    {
        anterior = atual;
        atual = atual->Proximo();
    }
    atual = new clNohLista<TValor>;
    atual->CopiarValor(val);
    if (anterior == NULL)
        primeiro = atual;
    else
        anterior->Proximo(atual);
}

// Imprime todos os elementos da lista na saida padrao
template <class TValor>
void clLista<TValor>::Imprimir()
{
    clNohLista<TValor> *noh;

    noh = primeiro;

    while (noh != NULL)
    {
        noh->Valor().Imprimir();
        noh = noh->Proximo();
    }
}

#endif

